/*******************************************************************************
 * Copyright (c) 2004 Berthold Daum. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Common
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors: Berthold Daum
 ******************************************************************************/

package com.sun.speech.freetts.en.us;

import java.io.IOException;
import java.net.URL;
import java.util.Locale;

import com.sun.speech.freetts.Age;
import com.sun.speech.freetts.Gender;
import com.sun.speech.freetts.UtteranceProcessor;
import com.sun.speech.freetts.relp.AnimatedAudioOutput;

/**
 * @author Berthold Daum
 *
 * Creation date: 01.10.2003
 * 
 */
public class AnimatedDiphoneVoice extends CMUDiphoneVoice {

	UtteranceProcessor animator;

	/**
	 * Constructor
	 * @param name - the name of the voice
	 * @param gender - the gender of the voice
	 * @param age - the age of the voice
	 * @param description - a human-readable string providing a
	 *            description that can be displayed to users.
	 * @param locale - the locale of the voice
	 * @param domain - the domain of this voice, e.g. general, time, wheather
	 * @param organization the organization which created the voice
	 * @param lexicon - the lexicon to load
	 * @param database - a url to the unit database file for this voice
	 * @param animator - the animator for lip synchronization
	 */
	public AnimatedDiphoneVoice(
		String name,
		Gender gender,
		Age age,
		String description,
		Locale locale,
		String domain,
		String organization,
		CMULexicon lexicon,
		URL database,
		UtteranceProcessor animator) {
		super(
			name,
			gender,
			age,
			description,
			locale,
			domain,
			organization,
			lexicon,
			database);
		this.animator = animator;
	}

	/**
	 * @see com.sun.speech.freetts.Voice#getAudioOutput()
	 */
	protected UtteranceProcessor getAudioOutput() throws IOException {
		return new AnimatedAudioOutput(animator);
	}

}